@@ -21,22 +21,33 @@
  * $Id: module_main.c,v 1.17 2008-10-15 17:05:37 hanke Exp $
  */
 
-#if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
-/* Added by Willie Walker - getline is a gcc-ism */
-ssize_t getline (char **lineptr, size_t *n, FILE *f);
+#ifdef HAVE_CONFIG_H
+#include <config.h>
 #endif
 
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <pthread.h>
+#include <glib.h>
+#include <dotconf.h>
+#include <ltdl.h>
+
+#include <spd_utils.h>
+#include "module_utils.h"
+
 #define PROCESS_CMD(command, function) \
 if (!strcmp(cmd_buf, #command"\n")){ \
  char *msg; \
  pthread_mutex_lock(&module_stdout_mutex); \
  if (printf("%s\n", msg = (char*) function()) < 0){ \
      DBG("Broken pipe, exiting...\n"); \
-     module_close(2); \
+     ret = 2; \
+     break; \
  } \
  fflush(stdout); \
  pthread_mutex_unlock(&module_stdout_mutex);\
- xfree(msg); \
+ g_free(msg); \
 }
 
 #define PROCESS_CMD_W_ARGS(command, function) \
@@ -45,19 +56,17 @@ if (!strncmp(cmd_buf, #command, strlen(#command))){	\
  pthread_mutex_lock(&module_stdout_mutex); \
  if (printf("%s\n", msg = (char*) function(cmd_buf)) < 0){ \
      DBG("Broken pipe, exiting...\n"); \
-     module_close(2); \
+     ret = 2; \
+     break; \
  } \
  fflush(stdout); \
  pthread_mutex_unlock(&module_stdout_mutex);\
- xfree(msg); \
+ g_free(msg); \
 }
 
 #define PROCESS_CMD_NRP(command, function) \
 if (!strcmp(cmd_buf, #command"\n")){ \
- pthread_mutex_lock(&module_stdout_mutex); \
  function(); \
- fflush(stdout); \
- pthread_mutex_unlock(&module_stdout_mutex);\
 }
 
 int 
@@ -65,83 +74,98 @@ main(int argc, char *argv[])
 {
     char *cmd_buf;
     int ret;
-    int ret_init;
     size_t n;
-    char *configfilename;
-    char *status_info;        
+    char *configfilename = NULL;
+    char *status_info = NULL;
 
     g_thread_init(NULL);
 
+    /* Initialize ltdl's list of preloaded audio backends. */
+    LTDL_SET_PRELOADED_SYMBOLS();
     module_num_dc_options = 0;
-    
+    module_audio_id = 0;
+
     if (argc >= 2){
-        configfilename = strdup(argv[1]);
-    }else{
-        configfilename = NULL;
+        configfilename = g_strdup(argv[1]);
     }
 
     ret = module_load();
-    if (ret == -1) module_close(1);
+    if (ret == -1) {
+	module_close();
+	exit(1);
+    }
 
     if (configfilename != NULL){
         /* Add the LAST option */
-        module_dc_options = add_config_option(module_dc_options,
+        module_dc_options = module_add_config_option(module_dc_options,
                                               &module_num_dc_options, "", 0, NULL, NULL, 0);
 
         configfile = dotconf_create(configfilename, module_dc_options, 0, CASE_INSENSITIVE);
         if (configfile){
             if (dotconf_command_loop(configfile) == 0){
                 DBG("Error reading config file\n");
-                module_close(1);
+                module_close();
+		exit(1);
             }
 	    dotconf_cleanup(configfile);
 	    DBG("Configuration (pre) has been read from \"%s\"\n", configfilename);    
 	    
-	    xfree(configfilename);
+	    g_free(configfilename);
         }else{
             DBG("Can't read specified config file!\n");        
         }
     }else{
         DBG("No config file specified, using defaults...\n");        
     }
-    
-    ret_init = module_init(&status_info);
 
     cmd_buf = NULL;  n=0;
-    ret = getline(&cmd_buf, &n, stdin);
+    ret = spd_getline(&cmd_buf, &n, stdin);
     if (ret == -1){
 	DBG("Broken pipe when reading INIT, exiting... \n");
-	module_close(2); 
+	module_close();
+	exit(2);
     }
 
-    if (!strcmp(cmd_buf, "INIT\n")){
-	if (ret_init == 0){
-	    printf("299-%s\n", status_info);
-	    ret = printf("%s\n", "299 OK LOADED SUCCESSFULLY");
-	}else{
-	    printf("399-%s\n", status_info);
-	    ret = printf("%s\n", "399 ERR CANT INIT MODULE");
-	    return -1;
-	}
-      	xfree(status_info);
-
-	if (ret < 0){ 
-	    DBG("Broken pipe, exiting...\n");
-            module_close(2); 
-	}
-	fflush(stdout);
-    }else{
+    if (strcmp(cmd_buf, "INIT\n")){
 	DBG("ERROR: Wrong communication from module client: didn't call INIT\n");
-	module_close(3);
+	module_close();
+	exit(3);
     }
-    xfree(cmd_buf);
+
+    ret = module_init(&status_info);
+
+    if (status_info == NULL) {
+        status_info = g_strdup("unknown, was not set by module");
+    }
+
+    if (ret != 0){
+	printf("399-%s\n", status_info);
+	printf("%s\n", "399 ERR CANT INIT MODULE");
+	g_free(status_info);
+	module_close();
+	exit(1);
+    }
+
+    printf("299-%s\n", status_info);
+    ret = printf("%s\n", "299 OK LOADED SUCCESSFULLY");
+
+    if (ret < 0){
+	DBG("Broken pipe, exiting...\n");
+        module_close();
+        exit(2);
+    }
+    fflush(stdout);
+
+    g_free(status_info);
+    g_free(cmd_buf);
 
     while(1){
         cmd_buf = NULL;  n=0;
-        ret = getline(&cmd_buf, &n, stdin);
+        ret = spd_getline(&cmd_buf, &n, stdin);
         if (ret == -1){
             DBG("Broken pipe, exiting... \n");
-            module_close(2); 
+	    ret = 2;
+            break;
         }
 
 	DBG("CMD: <%s>", cmd_buf);
@@ -157,14 +181,20 @@ main(int argc, char *argv[])
         else PROCESS_CMD(AUDIO, do_audio) 
         else PROCESS_CMD(LOGLEVEL, do_loglevel)
 	else PROCESS_CMD_W_ARGS(DEBUG, do_debug)
-        else PROCESS_CMD_NRP(QUIT, do_quit) 
+        else if (!strcmp(cmd_buf,"QUIT\n")) {
+	     do_quit();
+             exit(0);
+             }
         else{
           printf("300 ERR UNKNOWN COMMAND\n"); 
           fflush(stdout);
         }
 	
-	xfree(cmd_buf);
-    } 
+	g_free(cmd_buf);
+    }
+
+    module_close();
+    exit(ret);
 }
 
 #undef PROCESS_CMD
