@@ -21,95 +21,26 @@
  * $Id: module_utils.c,v 1.55 2008-07-10 15:37:18 hanke Exp $
  */
 
-#include "fdsetconv.h"
-#include "fdsetconv.c"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
+#if HAVE_SNDFILE
+#include <sndfile.h>
+#endif
+
+#include <fdsetconv.h>
+#include <spd_utils.h>
 #include "module_utils.h"
 
+static char* module_audio_pars[10];
 
 extern char* module_index_mark;
 
-#if !(defined(__GLIBC__) && defined(_GNU_SOURCE))
-/* Added by Willie Walker - getline is a gcc-ism */
-#define BUFFER_LEN 256
-ssize_t getline (char **lineptr, size_t *n, FILE *f)
-{
-        char ch;
-        size_t m = 0;
-        ssize_t buf_len = 0;
-        char * buf = NULL;
-        char * p = NULL;
-
-	if (errno != 0) {
-                DBG("getline: errno came in as %d!!!\n", errno);
-	        errno = 0;
-	}
-        while ( (ch = getc(f)) !=EOF )
-        {
-                if (errno != 0)
-                        return -1;
-                if ( m++ >= buf_len )
-                {
-                        buf_len += BUFFER_LEN;
-                        buf = (char *) realloc(buf, buf_len + 1);
-                        if ( buf == NULL )
-                        {
-                                DBG("buf==NULL");
-                                return -1;
-                        }
-                        p = buf + buf_len - BUFFER_LEN;
-                }
-                *p = ch;
-                p++;
-                if ( ch == '\n' )
-                        break;
-        }
-        if ( m == 0 )
-        {
-                DBG("getline: m=%d!",m);
-                return -1;
-        } else {
-                *p = '\0';
-                *lineptr = buf;
-                *n = m;
-                return m;
-        }
-}
-#endif /* !(defined(__GLIBC__) || defined(_GNU_SOURCE)) */
-
-void*
-xmalloc(size_t size)
-{
-    void *p;
-
-    p = malloc(size);
-    if (p == NULL) FATAL("Not enough memmory");
-    return p;
-}          
-
-void*
-xrealloc(void *data, size_t size)
-{
-    void *p;
-
-    if (data != NULL) 
-        p = realloc(data, size);
-    else 
-        p = malloc(size);
-
-    if (p == NULL) FATAL("Not enough memmory");
-
-    return p;
-}          
-
-void
-xfree(void *data)
-{
-    if (data != NULL) free(data);
-}
+pthread_mutex_t module_stdout_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 char*
-do_message(EMessageType msgtype)
+do_message(SPDMessageType msgtype)
 {
     int ret;
     char *cur_line;
@@ -125,63 +56,83 @@ do_message(EMessageType msgtype)
     while(1){
         cur_line = NULL;
         n = 0;
-        ret = getline(&cur_line, &n, stdin);
+        ret = spd_getline(&cur_line, &n, stdin);
         nlines++;
-        if (ret == -1) return strdup("401 ERROR INTERNAL");
+        if (ret == -1) return g_strdup("401 ERROR INTERNAL");
 
         if (!strcmp(cur_line, "..\n")){
-            xfree(cur_line);
-            cur_line = strdup(".\n");
+            g_free(cur_line);
+            cur_line = g_strdup(".\n");
         }else if (!strcmp(cur_line, ".\n")){
             /* Strip the trailing \n */
             msg->str[strlen(msg->str)-1] = 0;
-	    xfree(cur_line);
+	    g_free(cur_line);
             break;
         }
         g_string_append(msg, cur_line);
-	xfree(cur_line);
+	g_free(cur_line);
     }
     
 
-    if ((msgtype != MSGTYPE_TEXT) && (nlines > 2)){
-        return strdup("305 DATA MORE THAN ONE LINE");
+    if ((msgtype != SPD_MSGTYPE_TEXT) && (nlines > 2)){
+        return g_strdup("305 DATA MORE THAN ONE LINE");
     }
 
-    if ((msgtype == MSGTYPE_CHAR) && (!strcmp(msg->str,"space"))){
+    if ((msgtype == SPD_MSGTYPE_CHAR) && (!strcmp(msg->str,"space"))){
 	g_string_free(msg, 1);
 	msg = g_string_new(" ");
     }
 
+    /* no sure we need this check here at all */
+    if (msg->str == NULL || msg->str[0] == 0){
+	DBG("requested data NULL or empty\n");
+	g_string_free(msg, TRUE);
+	return g_strdup("301 ERROR CANT SPEAK");
+    }
+
+    /* check voice and synthesis_voice settings for consistency */
+    if (msg_settings.voice.name == NULL
+        && msg_settings_old.voice.name != NULL
+        && msg_settings.voice_type == msg_settings_old.voice_type) {
+        /* force to set voice again, since synthesis_voice changed to NULL */
+	msg_settings_old.voice_type = -1;
+    }
+
+    /* Volume is controlled by the synthesizer. Always play at normal on audio device. */
+    if (spd_audio_set_volume(module_audio_id, 85) < 0) {
+        DBG("Can't set volume. audio not initialized?");
+    }
+
     ret = module_speak(msg->str, strlen(msg->str), msgtype);
 
     g_string_free(msg,1);
-    if (ret <= 0) return strdup("301 ERROR CANT SPEAK");
+    if (ret <= 0) return g_strdup("301 ERROR CANT SPEAK");
     
-    return strdup("200 OK SPEAKING");
+    return g_strdup("200 OK SPEAKING");
 }
 
 char*
 do_speak(void)
 {
-    return do_message(MSGTYPE_TEXT);
+    return do_message(SPD_MSGTYPE_TEXT);
 }
 
 char*
 do_sound_icon(void)
 {
-    return do_message(MSGTYPE_SOUND_ICON);
+    return do_message(SPD_MSGTYPE_SOUND_ICON);
 }
 
 char*
 do_char(void)
 {
-    return do_message(MSGTYPE_CHAR);
+    return do_message(SPD_MSGTYPE_CHAR);
 }
 
 char*
 do_key(void)
 {
-    return do_message(MSGTYPE_KEY);
+    return do_message(SPD_MSGTYPE_KEY);
 }
 
 void
@@ -215,9 +166,9 @@ do_pause(void)
 
 #define SET_PARAM_STR(name) \
  if(!strcmp(cur_item, #name)){ \
-     xfree(msg_settings.name); \
+     g_free(msg_settings.name); \
      if(!strcmp(cur_value, "NULL")) msg_settings.name = NULL; \
-     else msg_settings.name = strdup(cur_value); \
+     else msg_settings.name = g_strdup(cur_value); \
  }
 
 #define SET_PARAM_STR_C(name, fconv) \
@@ -243,10 +194,10 @@ do_set(void)
 
     while(1){
         line = NULL; n = 0;
-        ret = getline(&line, &n, stdin);
+        ret = spd_getline(&line, &n, stdin);
         if (ret == -1){ err=1; break; }
         if (!strcmp(line, ".\n")){
-            xfree(line);
+            g_free(line);
             break;
         }
         if (!err){
@@ -261,37 +212,43 @@ do_set(void)
             else SET_PARAM_STR_C(punctuation_mode, str2EPunctMode)
             else SET_PARAM_STR_C(spelling_mode, str2ESpellMode)
             else SET_PARAM_STR_C(cap_let_recogn, str2ECapLetRecogn)
-            else SET_PARAM_STR_C(voice, str2EVoice)
-            else SET_PARAM_STR(synthesis_voice)
-            else SET_PARAM_STR(language)
+            else
+                if(!strcmp(cur_item, "voice")){
+                    ret = str2EVoice(cur_value);
+                    if (ret != -1) msg_settings.voice_type = ret;
+                    else err = 2;
+                }
+            else
+                if(!strcmp(cur_item, "synthesis_voice")){
+                    g_free(msg_settings.voice.name);
+                    if(!strcmp(cur_value, "NULL")) msg_settings.voice.name = NULL;
+                    else msg_settings.voice.name = g_strdup(cur_value);
+                }
+            else
+                if(!strcmp(cur_item, "language")){
+                    g_free(msg_settings.voice.language);
+                    if(!strcmp(cur_value, "NULL")) msg_settings.voice.language = NULL;
+                    else msg_settings.voice.language = g_strdup(cur_value);
+                }
             else err=2;             /* Unknown parameter */
         }
-        xfree(line);
+        g_free(line);
     }
 
-    if (err == 0) return strdup("203 OK SETTINGS RECEIVED");
-    if (err == 1) return strdup("302 ERROR BAD SYNTAX");
-    if (err == 2) return strdup("303 ERROR INVALID PARAMETER OR VALUE");
+    if (err == 0) return g_strdup("203 OK SETTINGS RECEIVED");
+    if (err == 1) return g_strdup("302 ERROR BAD SYNTAX");
+    if (err == 2) return g_strdup("303 ERROR INVALID PARAMETER OR VALUE");
     
-    return strdup("401 ERROR INTERNAL"); /* Can't be reached */
+    return g_strdup("401 ERROR INTERNAL"); /* Can't be reached */
 }
 
-#define SET_AUDIO_NUM(name, cond) \
+#define SET_AUDIO_STR(name,idx) \
  if(!strcmp(cur_item, #name)){ \
-     number = strtol(cur_value, &tptr, 10); \
-     if(!(cond)){ err = 2; continue; } \
-     if (tptr == cur_value){ err = 2; continue; } \
-     audio_settings.name = number; \
+     g_free(module_audio_pars[idx]); \
+     if(!strcmp(cur_value, "NULL")) module_audio_pars[idx] = NULL; \
+     else module_audio_pars[idx] = g_strdup(cur_value); \
  }
 
-#define SET_AUDIO_STR(name) \
- if(!strcmp(cur_item, #name)){ \
-     xfree(audio_settings.name); \
-     if(!strcmp(cur_value, "NULL")) audio_settings.name = NULL; \
-     else audio_settings.name = strdup(cur_value); \
- }
-
-
 char*
 do_audio(void)
 {
@@ -300,9 +257,8 @@ do_audio(void)
     char *line = NULL;
     int ret;
     size_t n;
-    int number; char *tptr;
     int err = 0;                /* Error status */
-    char *status;
+    char *status = NULL;
     char *msg;
 
     printf("207 OK RECEIVING AUDIO SETTINGS\n");
@@ -310,10 +266,10 @@ do_audio(void)
 
     while(1){
         line = NULL; n = 0;
-        ret = getline(&line, &n, stdin);
+        ret = spd_getline(&line, &n, stdin);
         if (ret == -1){ err=1; break; }
         if (!strcmp(line, ".\n")){
-            xfree(line);
+            g_free(line);
             break;
         }
         if (!err){
@@ -322,19 +278,19 @@ do_audio(void)
             cur_value = strtok(NULL, "\n");
             if (cur_value == NULL){ err=1; continue; }
             
-            SET_AUDIO_STR(audio_output_method)
-            else SET_AUDIO_STR(audio_oss_device)
-	    else SET_AUDIO_STR(audio_alsa_device)
-	    else SET_AUDIO_STR(audio_nas_server)
-            else SET_AUDIO_STR(audio_pulse_server)
-	    else SET_AUDIO_NUM(audio_pulse_min_length, 1)
+            SET_AUDIO_STR(audio_output_method, 0)
+            else SET_AUDIO_STR(audio_oss_device, 1)
+            else SET_AUDIO_STR(audio_alsa_device, 2)
+            else SET_AUDIO_STR(audio_nas_server, 3)
+            else SET_AUDIO_STR(audio_pulse_server, 4)
+            else SET_AUDIO_STR(audio_pulse_min_length, 5)
             else err=2;             /* Unknown parameter */
         }
-        xfree(line);
+        g_free(line);
     }
 
-    if (err == 1) return strdup("302 ERROR BAD SYNTAX");
-    if (err == 2) return strdup("303 ERROR INVALID PARAMETER OR VALUE");
+    if (err == 1) return g_strdup("302 ERROR BAD SYNTAX");
+    if (err == 2) return g_strdup("303 ERROR INVALID PARAMETER OR VALUE");
 
     err = module_audio_init(&status);
 
@@ -342,7 +298,8 @@ do_audio(void)
       msg = g_strdup_printf("203 OK AUDIO INITIALIZED");
     else
       msg = g_strdup_printf("300-%s\n300 UNKNOWN ERROR", status);
-    
+
+    g_free(status);
     return msg;
 }
 
@@ -352,7 +309,7 @@ do_audio(void)
      if(!(cond)){ err = 2; continue; } \
      if (tptr == cur_value){ err = 2; continue; } \
      log_level = number; \
-     spd_audio_set_loglevel(log_level); \
+     spd_audio_set_loglevel(module_audio_id, number); \
  }
 
 char*
@@ -365,7 +322,6 @@ do_loglevel(void)
     size_t n;
     int number; char *tptr;
     int err = 0;                /* Error status */
-    char *status;
     char *msg;
 
     printf("207 OK RECEIVING LOGLEVEL SETTINGS\n");
@@ -373,10 +329,10 @@ do_loglevel(void)
 
     while(1){
         line = NULL; n = 0;
-        ret = getline(&line, &n, stdin);
+        ret = spd_getline(&line, &n, stdin);
         if (ret == -1){ err=1; break; }
         if (!strcmp(line, ".\n")){
-            xfree(line);
+            g_free(line);
             break;
         }
         if (!err){
@@ -388,11 +344,11 @@ do_loglevel(void)
             SET_LOGLEVEL_NUM(log_level, 1)
             else err=2;             /* Unknown parameter */
         }
-        xfree(line);
+        g_free(line);
     }
 
-    if (err == 1) return strdup("302 ERROR BAD SYNTAX");
-    if (err == 2) return strdup("303 ERROR INVALID PARAMETER OR VALUE");
+    if (err == 1) return g_strdup("302 ERROR BAD SYNTAX");
+    if (err == 2) return g_strdup("303 ERROR INVALID PARAMETER OR VALUE");
 
     msg = g_strdup_printf("203 OK LOG LEVEL SET");
 
@@ -411,13 +367,13 @@ do_debug(char* cmd_buf)
 
   if (!cmd[1]){
     g_strfreev(cmd);
-    return strdup("302 ERROR BAD SYNTAX");
+    return g_strdup("302 ERROR BAD SYNTAX");
   }
 
   if (!strcmp(cmd[1], "ON")){
     if (!cmd[2]){
       g_strfreev(cmd);
-      return strdup("302 ERROR BAD SYNTAX");
+      return g_strdup("302 ERROR BAD SYNTAX");
     }
 
 
@@ -427,7 +383,7 @@ do_debug(char* cmd_buf)
     if (CustomDebugFile == NULL){
       DBG("ERROR: Can't open custom debug file for logging: %d (%s)",
 	  errno, strerror(errno));
-      return strdup("303 CANT OPEN CUSTOM DEBUG FILE");
+      return g_strdup("303 CANT OPEN CUSTOM DEBUG FILE");
     }
     if (Debug == 1)
       Debug = 3;
@@ -446,7 +402,7 @@ do_debug(char* cmd_buf)
     CustomDebugFile = NULL;
     DBG("Additional logging into specific path terminated");
   }else{
-    return strdup("302 ERROR BAD SYNTAX");
+    return g_strdup("302 ERROR BAD SYNTAX");
   }
 
   g_strfreev(cmd);
@@ -456,31 +412,44 @@ do_debug(char* cmd_buf)
 char *
 do_list_voices(void)
 {
-  VoiceDescription **voices;
+  SPDVoice **voices;
   int i;
-  char *lang, *dialect;
+  char *lang, *variant;
   GString *voice_list;
 
   voices = module_list_voices();
   if (voices == NULL){
-    return strdup("304 CANT LIST VOICES");
+    return g_strdup("304 CANT LIST VOICES");
   }
   
   voice_list = g_string_new("");
-  for (i=0; ;i++){
-    if (voices[i] == NULL) break;
-    if (voices[i]->language==NULL) lang=strdup("none");
-    else lang=strdup(voices[i]->language);
-    if (voices[i]->dialect==NULL) dialect=strdup("none");
-    else dialect=strdup(voices[i]->dialect);
-    g_string_append_printf(voice_list, "200-%s %s %s\n", voices[i]->name, lang, dialect);
-    xfree(lang); xfree(dialect);
+  for (i=0; voices[i] != NULL; i++){
+    if (voices[i]->name == NULL) {        /* Shouldn't happen! */
+      DBG("Unnamed voice found; ignoring it.");
+      continue;
+    }
+    if (voices[i]->language==NULL)
+      lang = "none";
+    else
+      lang = voices[i]->language;
+    if (voices[i]->variant==NULL)
+      variant= "none";
+    else
+      variant = voices[i]->variant;
+    g_string_append_printf(voice_list, "200-%s %s %s\n", voices[i]->name, lang, variant);
   }
+
+  /* check whether we found at least one voice */
+  if (voice_list->len == 0){
+    g_string_free(voice_list, TRUE);
+    return g_strdup("304 CANT LIST VOICES");
+  }
+
   g_string_append(voice_list, "200 OK VOICE LIST SENT");
 
-  DBG("Voice prepared to  sens to speechd");
+  DBG("Voice prepared to  send to speechd");
 
-  return voice_list->str;
+  return g_string_free(voice_list, FALSE);
 }
 
 #undef SET_PARAM_NUM
@@ -494,7 +463,11 @@ do_quit(void)
 {
     printf("210 OK QUIT\n");    
     fflush(stdout);
-    module_close(0);
+
+    spd_audio_close(module_audio_id);
+    module_audio_id = NULL;
+
+    module_close();
     return;
 }
 
@@ -593,7 +566,7 @@ module_strip_ssml(char *message)
     assert(message != NULL);
 
     len = strlen(message);
-    out = (char*) xmalloc(sizeof(char) * (len+1));
+    out = (char*) g_malloc(sizeof(char) * (len+1));
 
     for (i = 0, n = 0; i <= len; i++){
 
@@ -631,80 +604,8 @@ module_strip_punctuation_default(char *buf)
     module_strip_punctuation_some(buf, "~#$%^&*+=|<>[]_");
 }
 
-void
-module_speak_thread_wfork(sem_t *semaphore, pid_t *process_pid, 
-                          TChildFunction child_function,
-                          TParentFunction parent_function,
-                          int *speaking_flag, char **message, const EMessageType *msgtype,
-                          const size_t maxlen, const char *dividers, size_t *module_position,
-                          int *pause_requested)
-{
-    TModuleDoublePipe module_pipe;
-    int ret;
-    int status;
-
-    set_speaking_thread_parameters();
-
-    while(1){        
-        sem_wait(semaphore);
-        DBG("Semaphore on\n");
-
-        ret = pipe(module_pipe.pc);
-        if (ret != 0){
-            DBG("Can't create pipe pc\n");
-            *speaking_flag = 0;
-            continue;
-        }
-
-        ret = pipe(module_pipe.cp);
-        if (ret != 0){
-            DBG("Can't create pipe cp\n");
-            close(module_pipe.pc[0]);     close(module_pipe.pc[1]);
-            *speaking_flag = 0;
-            continue;
-        }
-
-	DBG("Pipes initialized");
-
-        /* Create a new process so that we could send it signals */
-        *process_pid = fork();
-
-        switch(*process_pid){
-        case -1:	
-            DBG("Can't say the message. fork() failed!\n");
-            close(module_pipe.pc[0]);     close(module_pipe.pc[1]);
-            close(module_pipe.cp[0]);     close(module_pipe.cp[1]);
-            *speaking_flag = 0;
-            continue;
-
-        case 0:
-            /* This is the child. Make it speak, but exit on SIGINT. */
-
-            DBG("Starting child...\n");
-            (* child_function)(module_pipe, maxlen);           
-            exit(0);
-
-        default:
-            /* This is the parent. Send data to the child. */
-
-            *module_position = (* parent_function)(module_pipe, *message, *msgtype,
-                                                   maxlen, dividers, pause_requested);
-
-            DBG("Waiting for child...");
-            waitpid(*process_pid, &status, 0);            
-            
-            *speaking_flag = 0;
-
-	    module_report_event_end();
-
-            DBG("child terminated -: status:%d signal?:%d signal number:%d.\n",
-                WIFEXITED(status), WIFSIGNALED(status), WTERMSIG(status));
-        }        
-    }
-}
-
 size_t
-module_parent_wfork(TModuleDoublePipe dpipe, const char* message, EMessageType msgtype,
+module_parent_wfork(TModuleDoublePipe dpipe, const char* message, SPDMessageType msgtype,
                     const size_t maxlen, const char* dividers, int *pause_requested)
 {
     unsigned int pos = 0;
@@ -716,7 +617,7 @@ module_parent_wfork(TModuleDoublePipe dpipe, const char* message, EMessageType m
 
     DBG("Entering parent process, closing pipes");
 
-    buf = (char*) malloc((maxlen+1) * sizeof(char));
+    buf = (char*) g_malloc((maxlen+1) * sizeof(char));
 
     module_parent_dp_init(dpipe);
 
@@ -814,18 +715,20 @@ module_child_dp_close(TModuleDoublePipe dpipe)
 void
 module_child_dp_write(TModuleDoublePipe dpipe, const char *msg, size_t bytes)
 {
+    int ret;
     assert(msg != NULL);
-    write(dpipe.cp[1], msg, bytes);       
+    ret = write(dpipe.cp[1], msg, bytes);
+    assert(ret);
 }
 
 int
 module_parent_dp_write(TModuleDoublePipe dpipe, const char *msg, size_t bytes)
 {
-    int ret;
+    ssize_t ret;
     assert(msg != NULL);
-    DBG("going to write %ld bytes", bytes);
+    DBG("going to write %lu bytes", (long unsigned) bytes);
     ret = write(dpipe.pc[1], msg, bytes);      
-    DBG("written %d bytes", ret);
+    DBG("written %ld bytes", (long) ret);
     return ret;
 }
 
@@ -833,7 +736,11 @@ int
 module_child_dp_read(TModuleDoublePipe dpipe, char *msg, size_t maxlen)
 {
     int bytes;
-    bytes = read(dpipe.pc[0], msg, maxlen);    
+    while ((bytes = read(dpipe.pc[0], msg, maxlen)) < 0) {
+        if (errno != EINTR) {
+            FATAL("Unable to read data");
+        }
+    }
     return bytes;
 }
 
@@ -841,7 +748,11 @@ int
 module_parent_dp_read(TModuleDoublePipe dpipe, char *msg, size_t maxlen)
 {
     int bytes;
-    bytes = read(dpipe.cp[0], msg, maxlen);    
+    while ((bytes = read(dpipe.cp[0], msg, maxlen)) < 0) {
+        if (errno != EINTR) {
+            FATAL("Unable to read data");
+        }
+    }
     return bytes;
 }
 
@@ -907,24 +818,6 @@ set_speaking_thread_parameters()
     pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);				
 }
 
-
-int
-module_write_data_ok(char *data)
-{
-    /* Tests */
-    if(data == NULL){
-        DBG("requested data NULL\n");		
-        return -1;
-    }
-
-    if(data[0] == 0){
-        DBG("requested data empty\n");
-        return -1;
-    }
-
-    return 0;
-}
-
 int
 module_terminate_thread(pthread_t thread)
 {
@@ -944,29 +837,12 @@ module_terminate_thread(pthread_t thread)
     return 0;
 }
 
-sem_t*
-module_semaphore_init()
-{
-    sem_t *semaphore;
-    int ret;
-
-    semaphore = (sem_t *) malloc(sizeof(sem_t));
-    if (semaphore == NULL) return NULL;
-    ret = sem_init(semaphore, 0, 0);
-    if (ret != 0){
-        DBG("Semaphore initialization failed");
-        xfree(semaphore);
-        semaphore = NULL;
-    }
-    return semaphore;
-}
-
 char *
 module_recode_to_iso(char *data, int bytes, char *language, char *fallback)
 {
     char *recoded;
     
-    if (language == NULL) recoded = strdup(data);
+    if (language == NULL) recoded = g_strdup(data);
 
     if (!strcmp(language, "cs"))
         recoded = (char*) g_convert_with_fallback(data, bytes, "ISO8859-2", "UTF-8",
@@ -980,24 +856,12 @@ module_recode_to_iso(char *data, int bytes, char *language, char *fallback)
     return recoded;
 }
 
-int
-semaphore_post(int sem_id)
-{
-    static struct sembuf sem_b;
-
-    sem_b.sem_num = 0;
-    sem_b.sem_op = 1;          /* V() */
-    sem_b.sem_flg = SEM_UNDO;
-    return semop(sem_id, &sem_b, 1);
-}
-
-
 void
 module_send_asynchronous(char *text)
 {
     pthread_mutex_lock(&module_stdout_mutex);
     DBG("Printing reply: %s", text);
-    fprintf(stdout, text);
+    fputs(text, stdout);
     fflush(stdout);
     DBG("Printed");
     pthread_mutex_unlock(&module_stdout_mutex);
@@ -1015,7 +879,7 @@ module_report_index_mark(char *mark)
     
     module_send_asynchronous(reply);
  
-    xfree(reply);
+    g_free(reply);
 }
 
 void
@@ -1056,8 +920,8 @@ module_add_config_option(configoption_t *options, int *num_options, char *name,
     assert(name != NULL);
 
     num_config_options++;
-    opts = (configoption_t*) realloc(options, (num_config_options+1) * sizeof(configoption_t));
-    opts[num_config_options-1].name = (char*) strdup(name);
+    opts = (configoption_t*) g_realloc(options, (num_config_options+1) * sizeof(configoption_t));
+    opts[num_config_options-1].name = (char*) g_strdup(name);
     opts[num_config_options-1].type = type;
     opts[num_config_options-1].callback = callback;
     opts[num_config_options-1].info = info;
@@ -1079,126 +943,118 @@ module_get_ht_option(GHashTable *hash_table, const char *key)
     return option;
 }
 
-
-configoption_t *
-add_config_option(configoption_t *options, int *num_config_options, char *name, int type,
-                  dotconf_callback_t callback, info_t *info,
-                  unsigned long context)
+int
+module_audio_init(char **status_info)
 {
-    configoption_t *opts;
+    char *error=0;
+    gchar **outputs;
+    int i=0;
+
+    DBG ("Openning audio output system");
+    if (NULL == module_audio_pars[0]) {
+        *status_info = g_strdup ("Sound output method specified in configuration not supported. "
+                                                 "Please choose 'oss', 'alsa', 'nas', 'libao' or 'pulse'.");
+        return -1;
+     }
+
+    outputs = g_strsplit(module_audio_pars[0], ",", 0);
+    while (NULL != outputs[i]) {
+        module_audio_id = spd_audio_open(outputs[i], (void**) &module_audio_pars[1], &error);
+        if (module_audio_id) {
+            DBG("Using %s audio output method", outputs[i]);
+            g_strfreev (outputs);
+	    *status_info = g_strdup("audio initialized successfully.");
+            return 0;
+        }
+        i++;
+     }
+
+    *status_info = g_strdup_printf("Opening sound device failed. Reason: %s. ", error);
+    g_free(error);	/* g_malloc'ed, in spd_audio_open. */
+
+    g_strfreev (outputs);
+    return -1;
 
-    (*num_config_options)++;
-    opts = (configoption_t*) realloc(options, (*num_config_options) * sizeof(configoption_t));
-    opts[*num_config_options-1].name = strdup(name);
-    opts[*num_config_options-1].type = type;
-    opts[*num_config_options-1].callback = callback;
-    opts[*num_config_options-1].info = info;
-    opts[*num_config_options-1].context = context;
-    return opts;
 }
 
 int
-module_utils_init(void)
+module_tts_output(AudioTrack track, AudioFormat format)
 {
-    /* Init mutex */
-    pthread_mutex_init(&module_stdout_mutex, NULL);
 
-    return 0;
-}
-
-
-#define ABORT(msg) g_string_append(info, msg); \
-	*status_info = info->str; \
-	g_string_free(info, 0); \
+    if (spd_audio_play(module_audio_id, track, format) < 0) {
+        DBG("Can't play track for unknown reason.");
 	return -1;
-
-int
-module_audio_init_spd(char **status_info)
-{
-  char * error;
-  GString *info;
-  char *module_audio_pars[10];
-  char *outputs;
-  int audio_output_set = 0;
-
-  info = g_string_new("");
-  outputs = audio_settings.audio_output_method;
-
-    DBG("Openning audio output system");
-    if (outputs[0]){
-	char *next = outputs;
-	size_t len;
-
-	do{
-	    outputs = next;
-	    next = strchr(outputs, ',');
-
-	    if (!outputs[0] || outputs[0] == ',')
-		continue;
-
-	    len = (next ? ((size_t)(next-outputs)) : strlen(outputs));
-
-	    if (len == 3 && strncmp("oss", outputs, len) == 0){
-		DBG("Using OSS audio output method");
-		module_audio_pars[0] = strdup(audio_settings.audio_oss_device);
-		module_audio_pars[1] = NULL;
-		module_audio_id = spd_audio_open(AUDIO_OSS, (void**) module_audio_pars, &error);
-		if (module_audio_id){
-		    module_audio_output_method = AUDIO_OSS;
-		    audio_output_set = 1;
-		}
-	    } else if (len == 5 && strncmp("libao", outputs, len) == 0){
-		DBG("Using libao audio output method");
-		module_audio_pars[0] = NULL;
-		module_audio_pars[1] = NULL;
-		module_audio_id = spd_audio_open(AUDIO_LIBAO, (void**) module_audio_pars, &error);
-		if (module_audio_id){
-		    module_audio_output_method = AUDIO_LIBAO;
-		    audio_output_set = 1;
-		}
-	    } else if (len == 4 && strncmp("alsa", outputs, len) == 0){
-		DBG("Using Alsa audio output method");
-		module_audio_pars[0] = audio_settings.audio_alsa_device;
-		module_audio_pars[1] = NULL;
-		module_audio_id = spd_audio_open(AUDIO_ALSA, (void**) module_audio_pars, &error);
-		if (module_audio_id){
-		    module_audio_output_method = AUDIO_ALSA;
-		    audio_output_set = 1;
-		    break;
-		}
-	    } else if (len == 3 && strncmp("nas", outputs, len) == 0){
-		DBG("Using NAS audio output method");
-		module_audio_pars[0] = audio_settings.audio_nas_server;
-		module_audio_pars[1] = NULL;
-		module_audio_id = spd_audio_open(AUDIO_NAS, (void**) module_audio_pars, &error);
-		if (module_audio_id){
-		    module_audio_output_method = AUDIO_NAS;
-		    audio_output_set = 1;
-		    break;
-		}
-	    } else if (len == 5 && strncmp("pulse", outputs, len) == 0){
-		DBG("Using PulseAudio output method");
-		module_audio_pars[0] = (void *) audio_settings.audio_pulse_server;
-		module_audio_pars[1] = (void *) audio_settings.audio_pulse_min_length;
-		module_audio_id = spd_audio_open(AUDIO_PULSE, (void**) module_audio_pars, &error);
-		if (module_audio_id){
-		    module_audio_output_method = AUDIO_PULSE;
-		    audio_output_set = 1;
-		    break;
-		}
-	    } else{
-		ABORT("Sound output method specified in configuration not supported. "
-		    "Please choose 'oss', 'alsa', 'nas' or 'pulse'.");
-	    }
-	} while(next++);
     }
-
-    if (!audio_output_set){
-	g_string_append_printf(info, "Opening sound device failed. Reason: %s. ", error);
-	ABORT("Can't open sound device.");
-    }
-
     return 0;
 }
+/* Plays the specified audio file. */
+int
+module_play_file(const char *filename)
+
+{
+	int result = 0;
+#if HAVE_SNDFILE
+	int subformat;
+	sf_count_t items;
+	sf_count_t readcount;
+	SNDFILE *sf;
+	SF_INFO sfinfo;
+
+	DBG("Playing |%s|", filename);
+	memset(&sfinfo, 0, sizeof(sfinfo));
+	sf = sf_open(filename, SFM_READ, &sfinfo);
+	if (NULL == sf) {
+		DBG("%s", sf_strerror(NULL));
+		return -1;
+	}
+	if (sfinfo.channels < 1 || sfinfo.channels > 2) {
+		DBG("ERROR: channels = %d.\n", sfinfo.channels);
+		result = FALSE;
+		goto cleanup1;
+	}
+	if (sfinfo.frames > 0x7FFFFFFF || sfinfo.frames == 0) {
+		DBG("ERROR: Unknown number of frames.");
+		result = FALSE;
+		goto cleanup1;
+	}
+
+	subformat = sfinfo.format & SF_FORMAT_SUBMASK;
+	items = sfinfo.channels * sfinfo.frames;
+	DBG("Frames = %ld, channels = %ld", sfinfo.frames,
+	    (long)sfinfo.channels);
+	DBG("Samplerate = %i, items = %Ld", sfinfo.samplerate,
+	    (long long)items);
+	DBG("Major format = 0x%08X, subformat = 0x%08X, endian = 0x%08X", sfinfo.format & SF_FORMAT_TYPEMASK, subformat, sfinfo.format & SF_FORMAT_ENDMASK);
+
+	if (subformat == SF_FORMAT_FLOAT || subformat == SF_FORMAT_DOUBLE) {
+		/* Set scaling for float to integer conversion. */
+		sf_command(sf, SFC_SET_SCALE_FLOAT_INT_READ, NULL, SF_TRUE);
+	}
+	AudioTrack track;
+	track.num_samples = sfinfo.frames;
+	track.num_channels = sfinfo.channels;
+	track.sample_rate = sfinfo.samplerate;
+	track.bits = 16;
+	track.samples = g_malloc(items * sizeof(short));
+	readcount = sf_read_short(sf, (short *)track.samples, items);
+	DBG("Read %Ld items from audio file.", (long long)readcount);
+
+	if (readcount > 0) {
+		track.num_samples = readcount / sfinfo.channels;
+		DBG("Sending %i samples to audio.", track.num_samples);
+		int ret = module_tts_output(track, SPD_AUDIO_LE);
+		if (ret < 0) {
+			DBG("ERROR: Can't play track for unknown reason.");
+			result = -1;
+			goto cleanup2;
+		}
+		DBG("Sent to audio.");
+	}
+cleanup2:
+	g_free(track.samples);
+cleanup1:
+	sf_close(sf);
+#endif
+	return result;
+}
 
-#undef ABORT
