@@ -19,17 +19,18 @@
  * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  *
- * $Id: compare.h,v 1.3 2006/07/11 16:12:27 hanke Exp $
+ * $Id: compare.h,v 1.3 2006-07-11 16:12:27 hanke Exp $
  */
 
 #ifndef COMPARE_H
- #define COMPARE_H
+#define COMPARE_H
 
-gint compare_message_fd (gconstpointer element, gconstpointer value, gpointer x);
-gint compare_message_uid (gconstpointer element, gconstpointer value, gpointer x);
+gint compare_message_fd(gconstpointer element, gconstpointer value, gpointer x);
+gint compare_message_uid(gconstpointer element, gconstpointer value,
+			 gpointer x);
 
 /* Pointers to functions compare_message_fd and compare_message_uid */
-gint (*p_msg_lc)();
-gint (*p_msg_uid_lc)();
+gint(*p_msg_lc) ();
+gint(*p_msg_uid_lc) ();
 
 #endif /* COMPARE_H */
